#!/bin/sh
################################################################################
# Name: common_install.sh
# Purpose: Portiopns of the install process common to all distributions
# Last edited: 8/23/00
# Command:
#    common_install.sh
# Input (via environment variables), set by calling program:
#    CLIENT_VENDOR - vendor name (RedHat, Caldera, TurboLinux)
#    CLIENT_DISTRIBUTION  (example: 6.1, WorkStation6.0, etc)
#    RPM_LIST_FILE - file name for client rpm list to install
################################################################################

PATH=$PATH:/sbin:/usr/sbin:/bin:/usr/bin

export DEFAULT_MOUNTPOINT=/tmp/nsinstall
export MOUNTPOINT=/nstation/install
function getNFSServer 
{
	echo -n "What is the NFS Install Server IP Address?: "; read -r NFSSERVER
	check_valid $NFSSERVER "getNFSServer"
}

function getMountPoint
{
	echo -n "What is the NFS Source Installation Directory?: "; read MOUNTPOINT
	if [ ! $MOUNTPOINT ]
		then 
			echo -e "\nNo value set.  Using defaults.\n"
			MOUNTPOINT=/nstation/install
	fi
}

function listMountPoints
{
	echo -e "\nAvailable mounts on: $NFSSERVER\n\n"
	LINECOUNT=`showmount -e $NFSSERVER | wc -l`
	TEMP=`showmount -e $NFSSERVER | grep -vE "Export list" | tr -s "*" " " | cut -f1 -d" " | tr -s "\n" "\t"`
	TEMPC=1	
	while [ $TEMPC -lt $LINECOUNT ]; do

                MountArray[$TEMPC]=`echo $TEMP | cut -f$TEMPC -d" " | cut -f1 -d" "`
		echo -e "\t $TEMPC. ${MountArray[$TEMPC]}"
		TEMPC=$((TEMPC+1))
	done
	echo -e "\t 0. Quit"
	echo -e "\n"
	echo -n "Choose a mount point: "; read -r SELECT
	checkForRange $SELECT $((LINECOUNT-1)) "listMountPoints"
	MOUNTPOINT=${MountArray[$SELECT]}
}


function unmountNFSServer
{
	echo -n "Unmounting Server..."
	umount $DEFAULT_MOUNTPOINT 
}

function mountNFSServer
{
	if [ $MOUNTPOINT ] && [ $NFSSERVER ]
		then 
			echo -e "\n\n"
			echo  "Mounting $NFSSERVER:$MOUNTPOINT  Please wait..."
 			if [ ! -d $DEFAULT_MOUNTPOINT ]; then
                           mkdir -p $DEFAULT_MOUNTPOINT
                        fi
			# Unmount any left over mounts, before mounting default_mountpoint
			umount $DEFAULT_MOUNTPOINT > /dev/null 2>&1 

			if mount $NFSSERVER:$MOUNTPOINT $DEFAULT_MOUNTPOINT
				then echo -e "Mount successful."
				else
					echo -e " ERROR: Mount unsuccessful. Check $NFSSERVER:$MOUNTPOINT:"
					unmountNFSServer
                                        
                        fi

		else
			echo -e "\n\n"
			echo -e "NFS server and mount point not both specified.\n"
			echo -e "Assuming $IMAGENAME is in file system.\n"
			USINGEXISTINGFS=true

	fi
	echo -e "\n"
}

function check_valid # number callingFunction 
{
	BADVAL=0
        TEMP=`echo $1 | tr -d "[a-z][A-Z][\-\"]"`
        if [ ${#TEMP} -ne ${#1} ]
                then BADVAL=1
        fi

        TEMP=`echo $1 | cut -f1 -d"."`
        if [ $TEMP -gt 255 ]
                then BADVAL=1
        fi
        TEMP=`echo $1 | cut -f2 -d"."`
        if [ $TEMP -gt 255 ]
                then BADVAL=1
        fi
        TEMP=`echo $1 | cut -f3 -d"."`
        if [ $TEMP -gt 255 ]
                then BADVAL=1
        fi
        TEMP=`echo $1 | cut -f4 -d"."`
        if [ $TEMP -gt 255 ]
                then BADVAL=1
        fi

	if [ $BADVAL -eq 1 ]
		then
			BADVAL=0
			echo -e "\nInvalid value\n";
			$2
	fi
} 

function checkForRange # number upperlimit callingFunction
{
	BADCHOICE=0
	if [ $1 -eq 0 ]
		then
			umount $DEFAULT_MOUNTPOINT
			sync;sync;sync
			exit 1
	fi

	if [ $1 -lt 1 ]
		then 
			BADCHOICE=1
	fi

	if [ $1 -gt $2 ]
		then
			BADCHOICE=1
	fi

	if [ $BADCHOICE -eq 1 ]
		then
			BADCHOICE=0
			echo -e "\nInvalid value.\n"
			$3
	fi
}



function NFS_install
{
        echo ""
        echo "Please select an option:"
        echo "  1. Change NFS Server IP Address [ $NFSSERVER ]"
        echo "  2. Change NFS Source Directory  [ $MOUNTPOINT ]"
        echo "  3. List exported directorys for $NFSSERVER."
        echo "  4. NFS Server IP Address and Source Directory are OK Continue."
        echo "  5. Quit - Abort installation."
        echo -n "Enter Selection: "; read
        case $REPLY in
                [1])    getNFSServer
                        NFS_install;;
                [2])    create_directory NFS
                        NFS_install;;  
                [3])    echo ""
                        showmount -e $NFSSERVER
                        NFS_install;;  
                [4])    ;;  
                [5])    exit 1;;  
                  *)
                        echo -e "\n\nInvalid Response\n\n"
                        NFS_install;;
        esac
}

function NFS_or_CDROM_install
{
	echo "Please select Installation type:"
	echo "  1. Local CDROM "
	echo "  2. NFS"
        echo -n "Enter Selection: "; read
	case $REPLY in
		[1])  create_directory CDROM;;
		[2])    getNFSServer
				  getMountPoint
#				  create_directory NFS
              NFS_install
                        mountNFSServer
			cdrom=$DEFAULT_MOUNTPOINT
                        ;;
		*)
			echo -e "\n\nInvalid Response\n\n"
			NFS_or_CDROM_install;;
	esac
}

function verifyInstallDir
{
	mountNFSServer
        if [  -d $DEFAULT_MOUNTPOINT/TurboLinux ]; then
                echo -e "Installation source directory is OK. \n"
        else
                echo -e "Check NFS Server Installation source directory and IP Address . \n"
                echo -n "Abort Installation [y] (y/n)?"; read input
	case $input in
		""|[yY]|[yY][eE][sS]) exit 1;;
		[nN]|[nN][oO])NFS_install;;
		*)
			echo -e "\n\nInvalid Response\n\n"
			verifyInstallDir;;
	esac
        fi
}


function create_directory
{
        case $1 in
                "NFS")
                        dir_prompt="Local Mount Point for NFS"
                        default_dir="/mnt/nfsinstall"
                        dir_var="cdrom";;
                "CDROM")
                        dir_prompt="Mount Point for CDROM"
                        default_dir="/mnt/cdrom"
                        dir_var="cdrom"
                ;;
                "CLIENT")
                        dir_prompt="Client Installation Directory"
                        default_dir="/nstation/linux"
                        dir_var="BOOT_DIR"
                ;;
                "MACHINE")
                        dir_prompt="Client Machine Specific Directory"
                        default_dir="/nstation/machines"
                        dir_var="MACHINE_BASE_DIR"
               ;;
               	 
        esac
        dir_done="false"
        while [ "$dir_done" = "false" ]; do
                echo -n "$dir_prompt [$default_dir] ? "; read dir_val
                if [ ! $dir_val ]; then
                        export dir_val=$default_dir
                else
                        case $dir_val in
                                [yY]|[yY][eE][sS])
                                        export dir_val=$default_dir;;          
                        esac
                fi
                if [ -d $dir_val ]; then
                        export $dir_var=$dir_val
                        dir_done="true"
							   # check for CLIENT AND MACHINES
							   # if they exist we want to remove them
							   if [ $1 = "CLIENT" ]; then 		
								    echo "Detected an existing installation."
 									 echo -n "Do you want to uninstall previous installation? (y/n) "; read ans
                            case $ans in
                               ""|[yY]|[yY][eE][sS])
                               rm -Rf $dir_val
										 mkdir $dir_val
                               ;;              
                               [nN]|[nN][oO]) 
										 echo "Aborting installation"
										 exit 1
										 ;;
                               *)
                               echo -e "\n\nInvalid Response\n\n"
                               continueQuestion;;
                            esac
								elif [ $1 = "MACHINE" ]; then
								   if [ -d $dir_val ]; then
                              rm -Rf $dir_val
										mkdir $dir_val
									fi

							   fi
							
                else
                        echo -n "$dir_prompt does not exist. Create $dir_val? "; read ans
                        case $ans in
                                ""|[yY]|[yY][eE][sS])
                                mkdir -p $dir_val
                                if [ -d $dir_val ]; then
                                        export $dir_var=$dir_val
                                        dir_done="true"
                                fi
                                ;;              
                                [nN]|[nN][oO]) ;;
                                *)
                                        echo -e "\n\nInvalid Response\n\n"
                                        continueQuestion;;
                        esac
                fi
        done
}


################################################################################
# continueQuestion
################################################################################
function continueQuestion
{
	echo -ne "\nDo you wish to continue, restart, or exit installation? (c/r/e) "
	read input
	case $input in
		[cC]|[Cc][Oo][Nn][Tt][Ii][Nn][Uu][Ee]) ;;
		[Rr]|[Rr][Ee][Ss][Tt][Aa][Rr][Tt]) main_common ;;
		[nN]|[nN][oO]|[eE]|[Ee][Xx][Ii][Tt]) exit 1;;
		*)
			echo -e "\n\nInvalid Response\n\n"
			continueQuestion;;
	esac
}

################################################################################
# preInstall_Verifications
################################################################################
function preInstall_Verifications
{


   # create a new tmp/bootDirValue file - holds boot, server, and other info 
   if [ -f tmp/bootDirValue ]; then
      rm -f tmp/bootDirValue
   fi
   touch tmp/bootDirValue

	# Be sure path contains local directory
	PATH=$PATH:.

	# If the current working directory is the same as the CDROM we should exit
	# as this can cause problems with mounting/unmounting the CDROMS
	if [ $CURDIR = $cdrom ]; then
		echo -e "\nDo not launch this installation from within the CDROM mount point.\n"
		exit 1
	fi

	# Prompt the end user for the server IP Address and verify it is a valid IP
	# getServerIP
	findServerIP

	# Prompt the end user for the server's Linux distribution
	serverDistribution

	# Prompt the end user for the installation directory
   create_directory CLIENT
	echo "BOOT_DIR=$BOOT_DIR" >> tmp/bootDirValue
}

################################################################################
# setMachineBaseDir
################################################################################
function setMachineBaseDir
{
  echo ""
  echo "In normal installations, the client defines a RAMdisk to hold temporary data."
  echo -n "Should the boot server hold client specific data (e.g. /tmp) [ No ] ? (Yes/No) "
  read input
  case $input in
    [yY]|[yY][eE][sS])
        create_directory MACHINE
      # Ask if temporary client data should be refreshed on every boot
      echo -n "Do you want client specific data persistent "
      echo -n "across client boot sessions [ No ] ? (Yes/No) "
      read persistent
      case $persistent in
        [yY]|[yY][eE][sS])
          MACHINE_DATA_PERSISTENT=y;;
        *)
          MACHINE_DATA_PERSISTENT=n;;
      esac
      ;;
    *)
      unset MACHINE_BASE_DIR;;
  esac
}

function setMachineBaseDirType
{
	echo ""
	echo -n "Machine specific directories based on MAC address or IP address [ MAC ] (IP/MAC) ? "
	read input
	case $input in 
		MAC)
			MACHINE_DIRECTORY_TYPE=MAC;;
		IP)
			MACHINE_DIRECTORY_TYPE=IP;;
		*)
			MACHINE_DIRECTORY_TYPE=MAC;;
	esac
}

function findServerIP
{
   interface=`route -n | grep '^0.0.0.0' | awk '{print $8}'`
   if [ ! $interface ]; then
      getServerIP
   fi

   IP=`/sbin/ifconfig $interface | grep "inet addr" | awk '{print $2}' | cut -d":" -f2`

   if [ ! $IP ]; then
      getServerIP
   fi

}

################################################################################
# getServerIP
################################################################################
function getServerIP
{

	while [ ! $validIP ]
	do
		echo -ne "\nServer IP Address: "; read IP
                if [ ! $IP ]; then
			if [ ! $validIP ]; then
				echo -e "\nInvalid value"; getServerIP;
			fi
                fi
		TEMP=`echo $IP | tr -d "[a-z][A-Z][\-\"]"`
		if [ ${#TEMP} -ne ${#IP} ]; then
			if [ ! $validIP ]; then
				echo -e "\nInvalid value"; getServerIP;
			fi
		fi
 
		TEMP=`echo $IP | cut -f1 -d"."`
		if [ $TEMP -gt 255 ]; then
			if [ ! $validIP ]; then
				echo -e "\nInvalid value"; getServerIP;
			fi
		fi
		TEMP=`echo $IP | cut -f2 -d"."`
		if [ $TEMP -gt 255 ]; then
			if [ ! $validIP ]; then
				echo -e "\nInvalid value"; getServerIP;
			fi
		fi
		TEMP=`echo $IP | cut -f3 -d"."`
		if [ $TEMP -gt 255 ]; then
			if [ ! $validIP ]; then
				echo -e "\nInvalid value"; getServerIP;
			fi
		fi
		TEMP=`echo $IP | cut -f4 -d"."`
		if [ $TEMP -gt 255 ]; then
			if [ ! $validIP ]; then
				echo -e "\nInvalid value"; getServerIP;
			fi
		fi
		validIP=1
	done
}

################################################################################
# serverDistribution
################################################################################
function serverDistribution
{
	CALDERA="Caldera Open Linux"
	REDHAT="RedHat Linux"
	SUSE="SuSe Linux"
	TURBO="TurboLinux"
	OTHER="Other"
	
	echo ""
	PS3="Which Linux distribution are you running on this server? "
	select i in "$CALDERA" "$REDHAT" "$SUSE" "$TURBO" "$OTHER"
	do
		case $i in
			"$CALDERA")
				DISTRIBUTION="$CALDERA"
				break;;
			"$REDHAT")
				DISTRIBUTION="$REDHAT"
				break ;;
			"$SUSE")
				DISTRIBUTION="$SUSE"
				break ;;
			"$TURBO")
				DISTRIBUTION="$TURBO"
				break ;;
			"$OTHER")
				DISTRIBUTION="$OTHER"
				break ;;
		esac
	done
	echo "SERVER=\"$DISTRIBUTION\"" >> tmp/bootDirValue

}

function setMachineServerType
{
	echo ""
	echo -e " 1) Thin clients authenticate to server (DEFAULT)"
	echo -e " 2) No authentication to server"
	echo -n "Select the mode you want for server booted clients? "
	read input
	case $input in 
		"1")
			MACHINE_TEMPLATE=NONKIOSK_SBOOT_TEMPLATE
		;;
		"2")
			MACHINE_TEMPLATE=KIOSK_SBOOT_TEMPLATE
		;;
		*)
			MACHINE_TEMPLATE=NONKIOSK_SBOOT_TEMPLATE
		;;
	esac
}

################################################################################
# Main Body
################################################################################
function main_common 
{
set +x
umask 022

# Determine if the current user is root - if not, exit
curUser=`whoami`
if [ $curUser != "root" ]; then
   echo -e "This install script must be run while logged in as root\n"
	return 0
fi

NFS_or_CDROM_install

preInstall_Verifications

echo ""
export cdrom



# ==========  Set Linux distribution specific Variables ===========================

# Default Settings
RC_BOOT_FILE=rc.sysinit
REBOOT_NEEDED=y

valid_client=n

# Specific setting for TurboLinux Client
if [ x$CLIENT_VENDOR = x"TurboLinux" ]; then
   valid_client=y
   REBOOT_NEEDED=y
   # Name of boot file to be modified
   RC_BOOT_FILE=rc.sysinit
   # RPM source directory (ends with slash)
   if [ ! $cdrom ]; then
      RPM_SOURCE_DIR=/mnt/cdrom/TurboLinux/RPMS/
   else
      RPM_SOURCE_DIR=$cdrom/TurboLinux/RPMS/
   fi
   RPM_ITERATIONS=2
fi
  
# Specific setting for RedHat Client
if [ x$CLIENT_VENDOR = x"RedHat" ]; then
   valid_client=y
   REBOOT_NEEDED=n
   # Name of boot file to be modified
   RC_BOOT_FILE=rc.sysinit
   # RPM source directory (ends with slash)
   if [ ! $cdrom ]; then
      RPM_SOURCE_DIR=/mnt/cdrom/RedHat/RPMS/
   else
      RPM_SOURCE_DIR=$cdrom/RedHat/RPMS/
   fi
   RPM_ITERATIONS=1
fi

# Specific setting for Caldera Client
if [ x$CLIENT_VENDOR = x"Caldera" ]; then
   valid_client=y
   echo "Support for Caldera not present"
   exit 1
fi

#check if its a valid client
if [ x$valid_client = x"n" ]; then 
   echo " "
   echo "ERROR: Invalid client vendor specified CLIENT_VENDOR = <$CLIENT_VENDOR>"
   echo "       Run the appropriate wrapper (e.g. tl_create_bootdir_from_rpm.sh)"
   echo "       Installation exiting"
   exit 1
fi

# Name of modified boot file
IBM_RC_BOOT_FILE=${RC_BOOT_FILE}.IBM_NS.sboot

# ==========  Set Common Variables applicable to client distributions ===========================

# Set the dir name of nc specific file - could be in tmp if tmp is a RAM disk, or could
# be in tmp.nc for nc specific files - Whatever is done here must match between 
# this script file and IBM_RC_BOOT_FILE (rc.sysinit.IBM_NS.sboot) file
# script
TMP_DIR=tmp

DEFAULT_MACHINE_BASE_DIR=/nstation/machines
MACHINE_TEMPLATE=NONKIOSK_SBOOT_TEMPLATE
MACHINE_DATA_PERSISTENT=y

#set MACHINE_BASE_DIR - directory that holds client-machine specific data (/tmp, var, etc)
MACHINE_BASE_DIR=$3

#if [ ! "$MACHINE_BASE_DIR" ]; then
#	setMachineBaseDir
#fi

# lets default to machine base dir by default. and persitent data...
create_directory MACHINE

setMachineBaseDirType

# determine if we set up server as kiosk or non-kiosk
setMachineServerType 


#current_dir=`pwd`

# Check if CDROM is mounted and files are present - CHECK if correct CD is present ???
if [ -d $RPM_SOURCE_DIR ]; then
  echo ""
else
  CDMOUNTED=`df | grep $cdrom | grep -v grep | wc -l`
  if [ $CDMOUNTED != 0 ]; then
    umount $cdrom
  fi
  CDMOUNTED=0
  while [ $CDMOUNTED = 0 ]
  do
    echo -e "\nPlease insert the $CLIENT_VENDOR $CLIENT_DISTRIBUTION CD"
    echo -n "   Press <ENTER> To Continue "; read
    echo ""
    CDMOUNTED=`df | grep $cdrom | grep -v grep | wc -l`
    if [ $CDMOUNTED = 0 ]; then
      mount $cdrom
      CDMOUNTED=`df | grep $cdrom | grep -v grep | wc -l`
    fi
    # Check if its the correct CD
    if [ ! -d $RPM_SOURCE_DIR ]; then
      umount $cdrom
      CDMOUNTED=0
    fi
  done
fi

# turbolinux - grep "^VERSION" version.cfg | cut -d'=' -f2
if [ $CLIENT_VENDOR = "TurboLinux" ]; then
   VERSION=`grep "^VERSION" $cdrom/version.cfg | cut -d'=' -f2`
   export CLIENT_DISTRIBUTION="$CLIENT_DISTRIBUTION $VERSION"
fi

if [ $BOOT_DIR = "/" ]; then
  echo -e "\nERROR: Attempting to install on the '/' directory - This will CORRUPT your server set-up"
  echo -e "         Installion terminating"
  exit 1
fi

echo "Installation of Server Boot $CLIENT_VENDOR $CLIENT_DISTRIBUTION Linux client for IBM NetVista thin client"
echo "Current selections: " 
echo "   Server IP address: " $IP
echo "   Client boot directory: " $BOOT_DIR
if [ x$MACHINE_BASE_DIR = x ]; then
  echo "   No Client SPECIFIC directory specified - Will use RAM disk for /tmp space"
else
  echo "   Client SPECIFIC directory: " $MACHINE_BASE_DIR
  echo "   Client SPECIFIC directory type: " $MACHINE_DIRECTORY_TYPE
#  echo "   Client data persistent across boot sessions: " $MACHINE_DATA_PERSISTENT
fi

# Check for existing installation
CLIENT_UPGRADE=n
if [ -d $BOOT_DIR/var.nc_shared ]; then
  echo ""
  echo "WARNING: Client boot directory set-up exists - This will upgrade the installation"
  CLIENT_UPGRADE=y
fi
#if [ -d $BOOT_DIR ]; then
#  echo ""
#  echo "WARNING: Client boot directory already exists - this will overwrite the previous installation"
#fi

# Be sure the administrator wants to continue the installation with these settings
continueQuestion

# shut down the nfs server
if [ -f /etc/rc.d/init.d/nfsserver ]; then
   /etc/rc.d/init.d/nfsserver stop
else
   /etc/rc.d/init.d/nfs stop
fi

# ============  Create infrastructure for RPM installation  =================
# create the boot dir - do we need to check is boot image has alredy been created
Common/scripts/mkdirhier ${BOOT_DIR}

./install_prep.sh $BOOT_DIR
# Create the directories, files needed for next step of RPM installation
if [ x$CLIENT_UPGRADE = x"y" ]; then 
  echo "Set-up for UPGRADE INSTALL "
  rm $BOOT_DIR/var
  mv $BOOT_DIR/var.nc_shared   $BOOT_DIR/var
else
  echo "Set-up for NEW INSTALL "
  cp -pR RPM_INSTALL_PREREQ/*   $BOOT_DIR
fi

# ==========  Perform distribution specific setup

if [ x$CLIENT_VENDOR = x"TurboLinux" ]; then
   #export the display - needed by some TurboLinux 6.0 Japan RPMs, like elisa*, kaname*, kanji18*, etc. 
   #   the default display of :0 does not work, so we set it to the server address
   # Check if this is needed for TL English version
   export DISPLAY=$IP:0.0
fi

# ============  Install RPMS from a shell script =================
if [ ! -f $RPM_LIST_FILE ]; then
   echo ERROR: No rpm list file $RPM_LIST_FILE present - Check the package
   exit 1
fi


./Common/scripts/makeRPMSlist.sh $RPM_LIST_FILE $RPM_SOURCE_DIR

echo "INSTALL RPMS - (THIS WILL TAKE APPROXIMATELY 10-15 MINUTES) ... "

rpm --initdb $BOOT_DIR/var/lib/rpm --root $BOOT_DIR

# pre-install of distribution specific RPMS like kernel and other
# rpms that we need to just update the database but not really install
if [ -f pre_install_RPMS.sh ]; then
  pre_install_RPMS.sh $BOOT_DIR
fi

rpm -ivh --ignorearch --excludedocs --root $BOOT_DIR `cat tmp/rpm.list` 2>tmp/rpm.err | tee tmp/rpm.log

rc=$?

if [ ! -f $BOOT_DIR/sbin/init ]; then
   # i couldn't find any good documenation on rpm return codes so 
   # i am assuming that if init isn't there we had a problem
   echo "rpm install failed with return code $rc"
   exit 1
fi

if [ -f post_install_RPMS.sh ]; then
   post_install_RPMS.sh $BOOT_DIR
fi

# remove /boot directory
rm -rf $BOOT_DIR/boot

echo ""

# ============ Create  /etc/fstab file  ========================
echo "Create " $BOOT_DIR"/etc/fstab file"
if [ -f $BOOT_DIR/etc/fstab.org ]; then
  echo $BOOT_DIR/etc/fstab.org file already exists
else
  if [ -f $BOOT_DIR/etc/fstab ]; then
    mv $BOOT_DIR/etc/fstab $BOOT_DIR/etc/fstab.org
  fi
fi
echo "## Generated for IBM Network Station " > $BOOT_DIR/etc/fstab
echo $IP:$BOOT_DIR " /  nfs  defaults  1 1" >> $BOOT_DIR/etc/fstab
echo "none   /proc  proc     defaults  0 0" >> $BOOT_DIR/etc/fstab
echo "none   /dev/pts   devpts  gid=5,mode=620  0 0" >> $BOOT_DIR/etc/fstab

# ============ Modify /etc/inittab file  ========================
echo "Modify $BOOT_DIR/etc/inittab to call $IBM_RC_BOOT_FILE"
if [ -f $BOOT_DIR/etc/inittab.org ]; then
  echo $BOOT_DIR/etc/inittab.org file already exists
else
  mv $BOOT_DIR/etc/inittab $BOOT_DIR/etc/inittab.org
fi
awk -fCommon/scripts/inittab_awk -vREPLACE=$RC_BOOT_FILE -vBY=$IBM_RC_BOOT_FILE  $BOOT_DIR/etc/inittab.org > $BOOT_DIR/etc/inittab

# For GUI-login - currently commented out, until we have menu options for GUI-login
#cp $BOOT_DIR/etc/inittab  $BOOT_DIR/etc/inittab.tmp
#awk -fCommon/scripts/inittab_awk  -vREPLACE="id:3:initdefault:" -vBY="id:5:initdefault:"  $BOOT_DIR/etc/inittab.tmp > $BOOT_DIR/etc/inittab

# =====================================================================================
#   Rename unneeded services  
# =====================================================================================
echo "Rename services not needed for IBM NetVista Thin Clients"
if [ -L $BOOT_DIR/etc/rc.d/init.d/linuxconf ]; then
  mv  $BOOT_DIR/etc/rc.d/init.d/linuxconf $BOOT_DIR/etc/rc.d/init.d/linuxconf.sav
fi
if [ -f $BOOT_DIR/etc/rc.d/init.d/gdm ]; then
  mv  $BOOT_DIR/etc/rc.d/init.d/gdm $BOOT_DIR/etc/rc.d/init.d/gdm.sav
fi
if [ -f $BOOT_DIR/etc/rc.d/init.d/junkbuster ]; then
  mv  $BOOT_DIR/etc/rc.d/init.d/junkbuster $BOOT_DIR/etc/rc.d/init.d/junkbuster.sav
fi
if [ -f $BOOT_DIR/etc/rc.d/init.d/xfs ]; then
  mv  $BOOT_DIR/etc/rc.d/init.d/xfs $BOOT_DIR/etc/rc.d/init.d/xfs.sav
fi
if [ -f $BOOT_DIR/etc/rc.d/init.d/atd ]; then
  mv  $BOOT_DIR/etc/rc.d/init.d/atd      $BOOT_DIR/etc/rc.d/init.d/atd.sav
fi
if [ -f $BOOT_DIR/etc/rc.d/init.d/apmd ]; then
  mv  $BOOT_DIR/etc/rc.d/init.d/apmd      $BOOT_DIR/etc/rc.d/init.d/apmd.sav
fi
if [ -f $BOOT_DIR/etc/rc.d/init.d/kudzu ]; then
  mv  $BOOT_DIR/etc/rc.d/init.d/kudzu    $BOOT_DIR/etc/rc.d/init.d/kudzu.sav
fi
if [ -f $BOOT_DIR/etc/rc.d/init.d/crond ]; then
  mv  $BOOT_DIR/etc/rc.d/init.d/crond    $BOOT_DIR/etc/rc.d/init.d/crond.sav
fi
if [ -f $BOOT_DIR/etc/rc.d/init.d/nfs ]; then
  mv  $BOOT_DIR/etc/rc.d/init.d/nfs    $BOOT_DIR/etc/rc.d/init.d/nfs.sav
fi
#if [ -f $BOOT_DIR/etc/rc.d/init.d/syslog ]; then
#  mv  $BOOT_DIR/etc/rc.d/init.d/syslog    $BOOT_DIR/etc/rc.d/init.d/syslog.sav
#fi
if [ -f $BOOT_DIR/etc/rc.d/init.d/sendmail ]; then
  mv  $BOOT_DIR/etc/rc.d/init.d/sendmail    $BOOT_DIR/etc/rc.d/init.d/sendmail.sav
fi

if [ -f $BOOT_DIR/etc/rc.d/init.d/random ]; then
  mv  $BOOT_DIR/etc/rc.d/init.d/random $BOOT_DIR/etc/rc.d/init.d/random.sav
fi



# =====================================================================================
#   Add new files
# =====================================================================================
echo "Add new Common Files "
cp -dpR Common/COMMON_FILES/*  $BOOT_DIR
echo "Add $CLIENT_VENDOR $CLIENT_DISTRIBUTION Specific Files "
cp -dpR SPECIFIC_FILES/*  $BOOT_DIR
# Force /tmp directory be RW - Some server setup (Caldera) has $BOOT_DIR/tmp set to RW for root only
chmod a+w $BOOT_DIR/tmp


# =====================================================================================
#   Link files directories to TMP_RAM_DISK - these are uniques file for each client
# =====================================================================================
echo "Link files to RAM disk"
# ==== Create link for var directory (created in RAM disk in /$TMP_DIR)  =======
#Link var directory - var created in ram disk under /$TMP_DIR
if [ -d $BOOT_DIR/var.nc_shared ]; then
  echo "Original var backup already present"
else
  echo "Backup var directory"
  mv $BOOT_DIR/var $BOOT_DIR/var.nc_shared
fi
ln -sf /$TMP_DIR/var $BOOT_DIR/var

#==== Link files used for networking
ln -sf /$TMP_DIR/HOSTNAME     $BOOT_DIR/etc/HOSTNAME
ln -sf /$TMP_DIR/hosts        $BOOT_DIR/etc/hosts
ln -sf /$TMP_DIR/resolv.conf  $BOOT_DIR/etc/resolv.conf
ln -sf /$TMP_DIR/network      $BOOT_DIR/etc/sysconfig/network

#==== Link files used for keyboard
ln -sf /$TMP_DIR/keyboard     $BOOT_DIR/etc/sysconfig/keyboard

#==== Link /etc/mtab file 
if [ -f $BOOT_DIR/etc/mtab ]; then
  rm $BOOT_DIR/etc/mtab
fi
ln -sf /$TMP_DIR/mtab $BOOT_DIR/etc/mtab

#==== Link loadable lib modules file 
rm -rf $BOOT_DIR/lib/modules
ln -sf /$TMP_DIR/lib-modules $BOOT_DIR/lib/modules


# link rpm dir on / to the actual - this is done to make server install of RPMS easy by
#   using the following command:  rpm -i --root BOOT_DIR <rpm-file-name>
#   Without the link, use "rpm -i --dbpath BOOT_DIR/var.nc_shared/lib/rpm <rpm-file-name>
#ln -sf var.nc_shared/lib/rpm $BOOT_DIR/rpm

# =====================================================================================
#   Perform any client specific operation
# =====================================================================================
if [ x$CLIENT_VENDOR = x"TurboLinux" ]; then
   # for TurboLinux 6.0.5, /etc/services.rpmnew needs to be renames as /etc/servcies
   if [ -f $BOOT_DIR/etc/services.rpmnew ]; then
      mv $BOOT_DIR/etc/services.rpmnew  $BOOT_DIR/etc/services
   fi
fi

# =====================================================================================
#   Set-up Xserver files for FrameBuffer or SVGA 
#   Make sure to change rc.IBM_NS.tmp to make the correct /tmp/XF86Config link
# =====================================================================================
XSERVER=SVGA
if [ $XSERVER = "SVGA" ]; then
  echo "Setup Xserver files - XF86_SVGA and XF86Config"
  # following line is for Japanese version only
  #  cp  Common/XSERVER_CHANGES/XF86Config.SVGA.jp    $BOOT_DIR/etc/X11/XF86Config.SVGA
  cp  Common/XSERVER_CHANGES/XF86Config.SVGA    $BOOT_DIR/etc/X11/XF86Config.SVGA
  cp  Common/XSERVER_CHANGES/XF86_SVGA          $BOOT_DIR/usr/X11R6/bin/
  if [ -f $BOOT_DIR/etc/X11/XF86Config.org ]; then
    echo "XF86Config.org already exists - no need to backup original"
  else
    if [ -f $BOOT_DIR/etc/X11/XF86Config ]; then
      mv $BOOT_DIR/etc/X11/XF86Config  $BOOT_DIR/etc/X11/XF86Config.org
    fi
  fi
  if [ -f $BOOT_DIR/etc/X11/XF86Config ]; then
    rm $BOOT_DIR/etc/X11/XF86Config
  fi
  ln -sf XF86Config.SVGA   $BOOT_DIR/etc/X11/XF86Config
  if [ -f $BOOT_DIR/etc/X11/X ]; then
    mv $BOOT_DIR/etc/X11/X $BOOT_DIR/etc/X11/X.org
  fi
  ln -sf /usr/X11R6/bin/XF86_SVGA  $BOOT_DIR/etc/X11/X             

else             

  ## For Frame buffer  ###
  echo "Setup Xserver files - XF86_FBDev and XF86Config"
#  cp  Common/XSERVER_CHANGES/XF86Config.FBDev.2800    $BOOT_DIR/etc/X11/
#  cp  Common/XSERVER_CHANGES/XF86Config.FBDev.2200    $BOOT_DIR/etc/X11/
  cp  Common/XSERVER_CHANGES/XF86Config.FBDev    $BOOT_DIR/etc/X11/
  if [ -f $BOOT_DIR/etc/X11/XF86Config.org ]; then
     echo "XF86Config.org already exists - no need to backup original"
  else
    if [ -f $BOOT_DIR/etc/X11/XF86Config ]; then
      mv $BOOT_DIR/etc/X11/XF86Config  $BOOT_DIR/etc/X11/XF86Config.org
    fi
  fi
  if [ -f $BOOT_DIR/etc/X11/XF86Config ]; then
    rm $BOOT_DIR/etc/X11/XF86Config
  fi
  ln -sf /$TMP_DIR/XF86Config  $BOOT_DIR/etc/X11/XF86Config
#  ln -sf XF86Config.FBDev.2800 $BOOT_DIR/etc/X11/XF86Config.2800
#  ln -sf XF86Config.FBDev.2200 $BOOT_DIR/etc/X11/XF86Config.2200

  # Link appropriate Xserver files
  if [ -f $BOOT_DIR/etc/X11/X ]; then
    mv $BOOT_DIR/etc/X11/X   $BOOT_DIR/etc/X11/X.org
  fi
  ln -sf /usr/X11R6/bin/XF86_FBDev  $BOOT_DIR/etc/X11/X             
  ln -sf /usr/X11R6/bin/XF86_FBDev  $BOOT_DIR/usr/X11R6/bin/X             

fi

# =====================================================================================
# Remove the unnecessary /dev entries from the BOOT_DIR tree
# =====================================================================================
#cp $BOOT_DIR/dev/MAKEDEV /tmp/
rm -rf $BOOT_DIR/dev/*
#mv /tmp/MAKEDEV $BOOT_DIR/dev/

# =====================================================================================
# Create machine specific dir ($MACHINE_BASE_DIR)
#  Template for this set-up is located in BOOT_DIR/IBM?TEMPLATES dir.
# =====================================================================================
if [ "$MACHINE_BASE_DIR" ]; then
   Common/scripts/mkdirhier  ${MACHINE_BASE_DIR}   
   echo "Machine specific template base directory" > ${MACHINE_BASE_DIR}/TEMPLATE_README
fi

# create /etc/rc.d/machineDir.IBM_NS.setup file contain MACHINE dir setup
# if $MACHINE_BASE_DIR is undefined, the file will have NULL value for MACHINE_BASE_DIR
echo MACHINE_BASE_DIR=$MACHINE_BASE_DIR > $BOOT_DIR/etc/rc.d/machineDir.IBM_NS.setup
echo MACHINE_TEMPLATE=$MACHINE_TEMPLATE >> $BOOT_DIR/etc/rc.d/machineDir.IBM_NS.setup
echo MACHINE_DATA_PERSISTENT=$MACHINE_DATA_PERSISTENT >> $BOOT_DIR/etc/rc.d/machineDir.IBM_NS.setup
echo MACHINE_DIRECTORY_TYPE=$MACHINE_DIRECTORY_TYPE >> $BOOT_DIR/etc/rc.d/machineDir.IBM_NS.setup

# =====================================================================================
# Generate /IBM/linuxDistribution (for version control) and /etc.motd files 
# =====================================================================================
. Common/version
echo -e "$CLIENT_VENDOR $CLIENT_DISTRIBUTION - IBM Version $linux_dist_ver ($linux_dist_date)\n" > $BOOT_DIR/IBM/linuxDistribution
echo -e "Welcome to $CLIENT_VENDOR $CLIENT_DISTRIBUTION - IBM Version $linux_dist_ver ($linux_dist_date)\n" > $BOOT_DIR/etc/motd
# Build $BOOT_DIR/IBM/version file
echo "linux_vendor=$CLIENT_VENDOR" > $BOOT_DIR/IBM/version
echo "linux_dist=$CLIENT_DISTRIBUTION" >> $BOOT_DIR/IBM/version
cat Common/version >> $BOOT_DIR/IBM/version


# =====================================================================================
# Export $BOOT_DIR
# =====================================================================================
temp=`grep $BOOT_DIR /etc/exports`
echo ""

if [ ! "$temp" ]; then
	echo "ETC_EXPORTS_MOD=y" >> tmp/bootDirValue
	if [ "$DISTRIBUTION" = "$REDHAT" ]; then
		echo "$BOOT_DIR		*(rw,no_root_squash)" >> /etc/exports
	elif [ "$DISTRIBUTION" = "$CALDERA" ]; then
		echo -n "Domain name for the clients which will be be booting from this server: "; read
		echo "$BOOT_DIR		*.$REPLY(rw,no_root_squash)" >> /etc/exports
	elif [ "$DISTRIBUTION" = "$TURBO" ]; then
		echo "$BOOT_DIR		*(rw,no_root_squash)" >> /etc/exports
		echo "The TurboLinux NFSd package relies upon the /etc/hosts.allow & /etc/hosts.deny"
		echo "files - if you have troubles booting your client, please consult the"
		echo "man pages for exports, hosts.allow, hosts.deny"
	elif [ "$DISTRIBUTION" = "$SUSE" ]; then
		echo "$BOOT_DIR		*(rw,no_root_squash)" >> /etc/exports
	elif [ "$DISTRIBUTION" = "$OTHER" ]; then
		echo "$BOOT_DIR		*(rw,no_root_squash)" >> /etc/exports
		echo "Your NFS server may require some configuration changes."
		echo "Consult the man pages for exports, hosts.allow, hosts.deny"
	fi

	echo "Added $BOOT_DIR to /etc/exports"
else
	echo "No change required for NFS export configuration"
fi

if [ "$MACHINE_BASE_DIR" ]; then
  temp=`grep $MACHINE_BASE_DIR /etc/exports`
  if [ ! "$temp" ]; then
	echo "ETC_EXPORTS_MOD=y" >> tmp/bootDirValue
	if [ "$DISTRIBUTION" = "$REDHAT" ]; then
		echo "$MACHINE_BASE_DIR		*(rw,no_root_squash)" >> /etc/exports
	elif [ "$DISTRIBUTION" = "$CALDERA" ]; then
		echo -n "Domain name for the clients which will be be booting from this server: "; read
		echo "$MACHINE_BASE_DIR		*.$REPLY(rw,no_root_squash)" >> /etc/exports
	elif [ "$DISTRIBUTION" = "$TURBO" ]; then
		echo "$MACHINE_BASE_DIR		*(rw,no_root_squash)" >> /etc/exports
		echo "The TurboLinux NFSd package relies upon the /etc/hosts.allow & /etc/hosts.deny"
		echo "files - if you have troubles booting your client, please consult the"
		echo "man pages for exports, hosts.allow, hosts.deny"
	elif [ "$DISTRIBUTION" = "$SUSE" ]; then
		echo "$MACHINE_BASE_DIR		*(rw,no_root_squash)" >> /etc/exports
	elif [ "$DISTRIBUTION" = "$OTHER" ]; then
		echo "$MACHINE_BASE_DIR		*(rw,no_root_squash)" >> /etc/exports
		echo "Your NFS server may require some configuration changes."
		echo "Consult the man pages for exports, hosts.allow, hosts.deny"
	fi

	echo "Added $MACHINE_BASE_DIR to /etc/exports"
  fi
fi

# =====================================================================================
# Copy the html documentation to the client tree for easy access
# =====================================================================================
cp -pR ../html $BOOT_DIR/IBM/

# =====================================================================================
# Extract the Flash Management Tutorial DOCS
# =====================================================================================
if [ -e /tmp/flash_manage_tutorial.tar.gz ]; then
	echo "Installing the Flash Management Tutorial documents in /IBM"
	tar -C $BOOT_DIR/IBM/ -xzf /tmp/flash_manage_tutorial.tar.gz
	rm -rf /tmp/flash_manage_tutorial.tar.gz
fi

# =====================================================================================
# chmod /usr/bin/smbmount for SMB home directory support
# =====================================================================================
if [ -f $BOOT_DIR/usr/sbin/smbmount ]; then
  chmod u+s $BOOT_DIR/usr/sbin/smbmount
fi

# make directory for idf's
mkdir -p $BOOT_DIR/mnt/update

umount $cdrom


# set up inittab
if [ $MACHINE_TEMPLATE = "KIOSK_SBOOT_TEMPLATE" ]; then
   cp $BOOT_DIR/IBM/Custom/inittab.kiosk-server.boot $BOOT_DIR/etc/inittab

   # create a kiosk home directory for system defaults
   mkdir -p $BOOT_DIR/home/system-default/.ICAClient
   cp -Rf $BOOT_DIR/IBM/MACHINE_DIR_TEMPLATES/KIOSK_SBOOT_TEMPLATE/tmp/home/KIOSK $BOOT_DIR/home/system-default
fi


# ====================================================================================
# Go through and strip anything that is not stripped (this might take a while)
# ====================================================================================
echo "Stripping binaries"
if [ -f strip.list ]; then
   for file in `cat strip.list`
   do
      strip $BOOT_DIR$file
   done
else
   find $BOOT_DIR -type f | xargs file | grep "not stripped" | grep Intel | grep -v "\.o:" | awk '{print $1}' | sed -n 's/://p' | xargs strip
fi


# create symlink from version of netscape installed to /usr/lib/netscape
NETSCAPE_VERSION=`ls -d $BOOT_DIR/usr/lib/netscape*`
if [ $NETSCAPE_VERSION != "$BOOT_DIR/usr/lib/netscape" ]; then
   LINK=`echo $NETSCAPE_VERSION | awk -F'/' '{print $NF}'`
   ln -s /usr/lib/$LINK $BOOT_DIR/usr/lib/netscape
fi

echo " "
echo "Client Directory Setup complete"
return 0
}
main_common
